using Jypeli;
using Jypeli.Assets;
using System;

namespace Demo7
{
    /// @author  Vesa Lappalainen
    /// @version 18.10.2021
    /// <summary>
    /// Peli, jossa vihaisia Legoja tiputellaan toisten päälle
    /// </summary>
    public class AngryLego : PhysicsGame
    {
        private static readonly String[] lines = {
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "/                       ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "        =======         ",
                  "        Y  Y  Y         ",
                  "        X* X  X         ",
                  "   *    X  X  X     *   ",
                  "        X  X  X         ",
                  "        X *X *X         ",
                  };

        private static readonly int tileWidth = 800 / lines[0].Length;
        private static readonly int tileHeight = 480 / lines.Length;

        private PhysicsObject maila;

        /// <summary>
        /// Aloitetaan peli. Aluksi siivotaan kaikki, jotta voidaan aloittaa uusi peli
        /// </summary>
        public override void Begin()
        {
            ClearGameObjects();
            ClearControls();
            Gravity = new Vector(0, -500);

            Level.Background.CreateGradient(Color.Blue, Color.White);
            TileMap tiles = TileMap.FromStringArray(lines);

            tiles.SetTileMethod('X', LuoSeina, Color.Wheat);
            tiles.SetTileMethod('Y', LuoYlaSeina, Color.Wheat);
            tiles.SetTileMethod('=', LuoKatto, Color.Red);
            tiles.SetTileMethod('/', LuoMaila, Color.Black);
            tiles.SetTileMethod('*', LuoVihollinen, Color.Pink);

            tiles.Execute(tileWidth, tileHeight);

            Level.CreateBorders();
            Camera.ZoomToLevel();

            Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Show help");
            Keyboard.Listen(Key.F5, ButtonState.Pressed, Begin, "New game");
            Keyboard.Listen(Key.Up, ButtonState.Pressed, KaannaMailaa, "Move up", 5.0);
            Keyboard.Listen(Key.Down, ButtonState.Pressed, KaannaMailaa, "Move down", -5.0);
            Keyboard.Listen(Key.Space, ButtonState.Pressed, PudotaPallo, "Drop ball");
        }


        /// <summary>
        /// Pudotetaan uusi pallo, joka voi rikkoa vihollisen
        /// </summary>
        private void PudotaPallo()
        {
            PhysicsObject pallo = new PhysicsObject(tileWidth, tileWidth, Shape.Circle);
            pallo.Color = Color.Yellow;
            pallo.Position = maila.Position + new Vector(0, maila.Height + tileWidth);
            pallo.Image = LoadImage("Igor");
            Add(pallo);
            AddCollisionHandler(pallo, "vihu", PalloOsui);
        }


        /// <summary>
        /// Apualiohjlema vihollisen räjäyttämiseksi ja poistamiseksi
        /// </summary>
        /// <param name="vihu"></param>
        private void PossautaVihu(IPhysicsObject vihu)
        {
            Explosion rajahdys = new Explosion(vihu.Width * 10);
            rajahdys.Position = vihu.Position;
            rajahdys.UseShockWave = false;
            Add(rajahdys);
            Remove(vihu);
        }


        /// <summary>
        /// Tapahtuma kun pallo osuu viholliseen
        /// </summary>
        /// <param name="pallo">pallo joka osui</param>
        /// <param name="vihu">vihollinen johon osuttiin</param>
        private void PalloOsui(PhysicsObject pallo, PhysicsObject vihu)
        {
            PossautaVihu(vihu);
        }


        /// <summary>
        /// Käännetään mailaa
        /// </summary>
        /// <param name="kulma">millä kulmalla käännetään</param>
        private void KaannaMailaa(double kulma)
        {
            maila.Angle += Angle.FromDegrees(kulma);
        }


        /// <summary>
        /// Luodaan seinäelementti
        /// </summary>
        private void LuoSeina(Vector paikka, double leveys, double korkeus, Color vari)
        {
            PhysicsObject seina = new PhysicsObject(leveys - 1, korkeus - 2);
            seina.Position = paikka;
            seina.Color = vari;
            seina.Tag = "rakenne";
            seina.Image = LoadImage("tiili");
            this.Add(seina);
        }


        /// <summary>
        /// Luodaan seinäelementti
        /// </summary>
        private void LuoYlaSeina(Vector paikka, double leveys, double korkeus, Color vari)
        {
            PhysicsObject seina = new PhysicsObject(leveys * 1.8, korkeus - 2);
            seina.Position = paikka;
            seina.Color = vari;
            seina.Tag = "rakenne";
            seina.Image = LoadImage("tiili");
            this.Add(seina);
        }


        /// <summary>
        /// Luodaan kattoelementti.  
        /// </summary>
        private void LuoKatto(Vector paikka, double leveys, double korkeus, Color vari)
        {
            PhysicsObject katto = new PhysicsObject(leveys - 1, korkeus - 1);
            katto.Position = paikka;
            katto.Color = vari;
            katto.Tag = "rakenne";
            Add(katto);
        }


        /// <summary>
        /// Luodaan maila, jolla palloja lyödään
        /// </summary>
        private void LuoMaila(Vector paikka, double leveys, double korkeus, Color vari)
        {
            maila = PhysicsObject.CreateStaticObject(leveys * 6, korkeus);
            maila.Position = paikka;
            maila.Color = vari;
            maila.Image = LoadImage("maila3");
            Add(maila);
        }


        /// <summary>
        /// Luodaan vihollinen, joka hajoaa osuessaan rekenteeseen
        /// </summary>
        private void LuoVihollinen(Vector paikka, double leveys, double korkeus, Color vari)
        {
            PhysicsObject vihu = new PhysicsObject(leveys / 2, leveys / 2, Shape.Circle);
            vihu.Position = paikka;
            vihu.Color = vari;
            vihu.Tag = "vihu";
            vihu.Image = LoadImage("Baby");
            AddCollisionHandler(vihu, "rakenne", VihuunOsui);
            Add(vihu);
        }


        /// <summary>
        /// Kun vihollinen osuu rakenteeseen
        /// </summary>
        /// <param name="vihu">vihollinen joka törmäsi</param>
        /// <param name="rakenne">rakenne johon osui</param>
        private void VihuunOsui(PhysicsObject vihu, PhysicsObject rakenne)
        {
            PossautaVihu(vihu);
        }
    }

}