using System;
using Demo9;

/// @author Aleksi Tarvainen
/// @version 2.11.2019
/// <summary>
/// Värienvaihtotehtävä ilman Jypeliä
/// </summary>
public class VarienVaihtoKonsolilla
{
    /// <summary>
    /// Vaihdetaan liian siniset pikselit harmaiksi 
    /// </summary>
    // BYCODEBEGIN
    public static void Main()
    {
        uint[,] kuva = {
            { 0xff767f73, 0xff838657, 0xff8e8937, 0xff9a8d67, 0xff91a6ac },
            { 0xff537888, 0xff65707e, 0xff65707e, 0xff838786, 0xff8e9da2 },
            { 0xff3a5e71, 0xff524c56, 0xff475f6e, 0xff537581, 0xff5c8697 }
        };
        Console.WriteLine("Ennen muunnosta:");
        TulostaKuva(kuva);
        // Poista kommentit
        // byte raja = 130;
        // SininenHarmaaksi(kuva, raja);
        Console.WriteLine("\nMuunnoksen jälkeen:");
        TulostaKuva(kuva);
    }

    
    // Toteuta tarvittava aliohjema tähän
    // BYCODEEND

    
    private static void TulostaKuva(uint[,] kuva)
    {
        for (int i = 0; i < kuva.GetLength(0); i++)
            for (int j = 0; j < kuva.GetLength(1); j++)
            {
                uint pikseli = kuva[i, j];
                byte blue = Color.GetBlue(pikseli);
                byte red = Color.GetRed(pikseli);
                byte green = Color.GetGreen(pikseli);
                byte alpha = Color.GetAlpha(pikseli);
                Console.WriteLine($"[{i}, {j}]: r: {red,3} g: {green,3} b: {blue,3} a: {alpha,3}");
            }
    }

}

namespace Demo9
{
    /// @author Aleksi Tarvainen
    /// @version 2.11.2019
    /// <summary>
    /// Pikkuinen luokka värien käsittelyyn Jypelin rajapintaa ja Vesan esimerkkejä
    /// mukaillen
    /// </summary>
    public class Color
    {
        /// <summary>
        /// Ottaa punaisen komponentin
        /// </summary>
        /// <param name="color">Väri etumerkittömänä kokonaislukuna</param>
        /// <returns>Punaisen määrä värissä</returns>
        public static byte GetRed(uint color)
        {
            return (byte)((color >> 16) & 0xff);
        }

        /// <summary>
        /// Ottaa vihreän komponentin väristä
        /// </summary>
        /// <param name="color">Väri etumerkittömänä kokonaislukuna</param>
        /// <returns>Vihreän määrä värissä</returns>
        public static byte GetGreen(uint color)
        {
            return (byte)((color >> 8) & 0xff);
        }

        /// <summary>
        /// Ottaa sinisen komponentin väristä
        /// </summary>
        /// <param name="color">Väri etumerkittömänä kokonaislukuna</param>
        /// <returns>Sinisen määrä värissä</returns>
        public static byte GetBlue(uint color)
        {
            return (byte)(color & 0xff);
        }

        /// <summary>
        /// Ottaa värin alpha-arvon eli läpinäkyvyyden
        /// </summary>
        /// <param name="color">Väri etumerkittömänä kokonaislukuna</param>
        /// <returns>Läpinäkyvyyden määrä värissä</returns>
        public static byte GetAlpha(uint color)
        {
            return (byte)((color >> 24) & 0xff);
        }

        /// <summary>
        /// Muodostaa yksittäisitä värikomponenteista värin etumerkittömänä 
        /// kokonaislukuna
        /// </summary>
        /// <param name="r">Punainen komponentti</param>
        /// <param name="g">Vihreä komponentti</param>
        /// <param name="b">Sininen komponentti</param>
        /// <param name="a">Alpha-arvo</param>
        /// <returns>Väri etumerkittömänä kokonaislukuna</returns>
        public static uint PackRGB(byte r, byte g, byte b, byte a = 0xff)
        {
            uint color = b;
            color |= (uint)(g << 8);
            color |= (uint)(r << 16);
            color |= (uint)(a << 24);
            return color;
        }
    }
}
